// Amanda Bischoff
// C++/NSL code for compilation of Bischoff model
// Began coding: 7 July 1994
// See <a href="../bgmvt.html">Bischoff Model Project Diary</a> and 
// <a href="../models.html">Bischoff Basal Ganglia Model</a> for a complete discussion 
// of the model

// include file definitions
#include "nsl_include.h"
#include <iostream.h>
#include <fstream.h>
#define NO_COLS 10

// <a name="inits"> <b>NSL Data Type Definitions</b></a>
NETWORK(BASAL_GANGLIA);

// note: lower case region names denote the membrane potential of the region
//       upper case region names denote firing rate (output) of the region
//       a 4 row matrix represents the following:
//	   row 0: flexion preparatory activity
//         row 1: flexion movement activity
//         row 2: extension preparatory activity
//         row 3: extension movement activity

// input to stimulate sma_flex_prep == PMC?
MATRIX(INPUT, 1, 10);
VECTOR(INPUT_TMP, 10);
MATRIX(INPUT_COL, 10, 1); // for displaying temporal graphs

// supplementary motor area
MATRIX(sma, 4, 10);
MATRIX(SMA, 4, 10);

// motor cortex
MATRIX(mc, 4, 10);
MATRIX(MC, 4, 10);

// putamen
MATRIX(put, 4, 10);
MATRIX(PUT, 4, 10);

// external globus pallidus 
MATRIX(gpe, 4, 10);
MATRIX(GPE, 4, 10);

// subthalamic nucleus
MATRIX(stn, 4, 10);
MATRIX(STN, 4, 10);

// internal globus pallidus
MATRIX(gpi, 4, 10);
MATRIX(GPI, 4, 10);

// substantia nigra pars reticulata
MATRIX(snr, 4, 10);
MATRIX(SNR, 4, 10);

// ventrolateral pars oralis thalamus
MATRIX(vlo, 4, 10);
MATRIX(VLO, 4, 10);

// The purpose of this vector is to simply indicate if a flexion (left box)
// or an extension (right box) was made.  It relies on input from both the
// motor cortex and the thalamus in order to perform the movement.
VECTOR(mvt, 2);
VECTOR(MVT, 2);
MATRIX(MVT_COL, 2, 1); // for temporal graph display only

// time constants: values provided in bgmvt.nsl
DATA(sma_tm);
DATA(mc_tm);
DATA(put_tm);
DATA(gpe_tm);
DATA(stn_tm);
DATA(gpi_tm);
DATA(snr_tm);
DATA(vlo_tm);
DATA(mvt_tm);

// masks of weights to demonstrate convergence of projections: 
//   values provided in bgmvt.nsl.  Convergence rate is not
//   at this time accurate, since according to Alexander (to appear),
//   for instance, cortical projections to a single putamen neuron is 1000:1!

// sma to putamen projections
MATRIX(sma_put_wts, 1, 10); // actual is 1000:1
// sma to sma projections (pre- and sma proper?)
MATRIX(sma_sma_wts, 3, 10); 
// first row is for mvt to prep cell connections
// second row is for mutual inhibition
// third row is for prep to mvt cell connections
// motor cortex to putamen projections
MATRIX(mc_put_wts, 1, 10); // actual is 1000:1
// sma to motor cortex projections
MATRIX(sma_mc_wts, 1, 10); // actual is ?
// motor cortex to sma projections
MATRIX(mc_sma_wts, 1, 10); // actual is ?
// putamen to gpe projections
MATRIX(put_gpe_wts, 1, 10); // actual is?
// gpe to stn projections
MATRIX(gpe_stn_wts, 1, 10);
// stn to gpi projections
MATRIX(stn_gpi_wts, 1, 10);
// putamen to gpi projections
MATRIX(put_gpi_wts, 1, 10);
// stn to snr projections
MATRIX(stn_snr_wts, 1, 10);
// putamen to snr projections
MATRIX(put_snr_wts, 1, 10);
// snr to thalamus projections
MATRIX(snr_vlo_wts, 1, 10);
// gpi to thalamus projections
MATRIX(gpi_vlo_wts, 1, 10);
// thalamus to motor cortex projections
MATRIX(vlo_mc_wts, 1, 10);
// thalamus to sma projections
MATRIX(vlo_sma_wts, 1, 10);
// motor cortex to thalamus projections
MATRIX(mc_vlo_wts, 1, 10);
// sma to thalamus projections
MATRIX(sma_vlo_wts, 1, 10);
// motor cortex to mvt vector
MATRIX(mc_mvt_wts, 1, 10);
// thalamus to mvt vector
MATRIX(vlo_mvt_wts, 1, 10);

// these weight matrices are used to scale up the value of the input, such
// that sma may be activated accordingly.  This was done because thalamic
// input to sma is at the ventrolateral thalamus may fire anywhere from about 5
// to 32 Hz (Vitek et al., 1994)
DATA(input_direction_wts);
DATA(input_center_wts);

// constants for use with NSLramp, specifying the following:
// kx1 -- range value at x; must be greater than this to turn "on"
// ky1 -- output value if sum < kx1
// ky2 -- minimum output value if sum >= kx1

// supplementary motor area
DATA(sma_kx1);
DATA(sma_ky1);
DATA(sma_ky2);
// motor cortex
DATA(mc_kx1);
DATA(mc_ky1);
DATA(mc_ky2);
// putamen
DATA(put_kx1);
DATA(put_ky1);
DATA(put_ky2);
// external globus pallidus 
DATA(gpe_kx1);
DATA(gpe_ky1);
DATA(gpe_ky2);
// subthalamic nucleus
DATA(stn_kx1);
DATA(stn_ky1);
DATA(stn_ky2);
// internal globus pallidus
DATA(gpi_kx1);
DATA(gpi_ky1);
DATA(gpi_ky2);
// substantia nigra pars reticulata
DATA(snr_kx1);
DATA(snr_ky1);
DATA(snr_ky2);
// ventrolateral thalamus
DATA(vlo_kx1);
DATA(vlo_ky1);
DATA(vlo_ky2);
// movement indicator
DATA(mvt_kx1);
DATA(mvt_ky1);
DATA(mvt_ky2);

// for tonic activity, known to exist in these regions
// external globus pallidus
DATA(gpe_tonic);
// subthalamic nucleus
DATA(stn_tonic);
// internal globus pallidus
DATA(gpi_tonic);
// substantia nigra pars reticulata
DATA(snr_tonic);
// ventrolateral thalamus
DATA(vlo_tonic);
// technically, VLo does not contain tonic neurons, but since I'm not yet
// putting in cortical input, I need something to counteract the inhibition of
// SNr/GPi on VLo.

// for putamen neurons, in determination of how long spiny neuron "bursts",
// and the negative hyperpolarization that occurs.  See comments in the
// <a href="#put">putamen</a> run module for further explanation.

// for preparatory flexion activity
DATA(burst_time_prep_flex);
DATA(put_hyperpol);
DATA(time_passed_prep_flex);
double delta;
DATA(put_flex_prep_on); // for use in determining if putamen has become active

// for movement flexion activity
DATA(burst_time_mvt_flex);
DATA(time_passed_mvt_flex);
DATA(put_flex_mvt_on); // for use in determining if putamen has become active

// for preparatory extension activity
DATA(burst_time_prep_extend);
DATA(time_passed_prep_extend);
DATA(put_extend_prep_on); // for use in determining if putamen has become active

// for movement extension activity
DATA(burst_time_mvt_extend);
DATA(time_passed_mvt_extend);
DATA(put_extend_mvt_on); // for use in determining if putamen has become active

// temporary vectors, needed in calculation of input to supplementary motor area
// as well as for the putamen.  More must be added if this program were running
// the run_modules in parallel.
VECTOR(temp0, 10);
VECTOR(temp1, 10);
VECTOR(temp2, 10);
VECTOR(temp3, 10);
VECTOR(test, 10);
// temporary vectors for computing the MVT cells, i.e., when a flexion or
// extension is being made
VECTOR(temp4,10);
VECTOR(temp5,10);

// end variable initialization

////////////////////////////////////////////////////////////////////////////////
// Initialization module
// The initial values chosen here reflect activity when there is no input to
// the system.  I got these by just letting the model run for a while, and it
// settled to these positions.  For regions with tonic activity (gpe, gpi, stn,
// snr, vlo), these values fall within the accepted range of activity of these
// regions.
////////////////////////////////////////////////////////////////////////////////
INIT_MODULE(bg_init_flex) {
  INPUT = 0; INPUT_TMP=INPUT.get_row(0); INPUT_COL = INPUT_COL.set_col(INPUT_TMP, 0);

  sma = 3.23361; mc = 1.0843; put = 0; gpe = 30; 
  stn = 12.59; gpi = 53.795; 
  snr = 51.7285; vlo = 13.843;
  mvt = 0;
  MVT = NSLstep(mvt, mvt_kx1, mvt_ky1, mvt_ky2);
  MVT_COL = MVT_COL.set_col(MVT, 0);
  SMA = NSLramp(sma, sma_kx1, sma_ky1, sma_ky2); 
  MC = NSLramp(mc, mc_kx1, mc_ky1, mc_ky2);
  PUT = NSLstep(put, put_kx1, put_ky1, put_ky2);
  burst_time_prep_flex = 0; burst_time_mvt_flex = 0;
  time_passed_prep_flex = 0; time_passed_mvt_flex = 0;
  burst_time_prep_extend = 0; burst_time_mvt_extend = 0;
  time_passed_prep_extend = 0; time_passed_mvt_extend = 0;
  delta = BASAL_GANGLIA.get_delta();
  put_flex_prep_on = put_kx1;
  put_flex_mvt_on = put_kx1;
  put_extend_prep_on = put_kx1;
  put_extend_mvt_on = put_kx1;
  GPE = NSLramp(gpe, gpe_kx1, gpe_ky1, gpe_ky2);
  STN = NSLramp(stn, stn_kx1, stn_ky1, stn_ky2);
  GPI = NSLramp(gpi, gpi_kx1, gpi_ky1, gpi_ky2);
  SNR = NSLramp(snr, snr_kx1, snr_ky1, snr_ky2);
  VLO = NSLramp(vlo, vlo_kx1, vlo_ky1, vlo_ky2);
}


// <a name="sma"><b>Supplementary Motor Area </b></a>
// <b>Click <a href="../model.html#sma">here</a> to read about assumptions.</b>
// Briefly, sma was broken up into the four vectors to calculate activity,
// since they get input from different areas.  This is to represent the
// different commands that it might be receiving in regards to a flexion or
// extension movement, and as to whether it should actually make the movement
// or merely prepare for the ensuing movement to be made.
RUN_MODULE(bg_sma) {
  temp0 = sma.get_row(0);
  temp1 = sma.get_row(1);
  temp2 = sma.get_row(2);
  temp3 = sma.get_row(3);
   INPUT_TMP=INPUT.get_row(0); INPUT_COL = INPUT_COL.set_col(INPUT_TMP,0);
// flexion preparatory
  DIFF.eq(temp0, sma_tm) = -sma.get_row(0) + INPUT.elem(0,4) * input_center_wts
    + NSLconv_copy(vlo_sma_wts.get_row(0), VLO.get_row(0))
    + NSLconv_copy(sma_sma_wts.get_row(0), SMA.get_row(1)) // from flex mvt cells
    - NSLconv_copy(sma_sma_wts.get_row(1), SMA.get_row(2)); // from prep ext cells
//    + NSLconv_copy(mc_sma_wts.get_row(0), MC.get_row(0));

// flexion movement
  DIFF.eq(temp1, sma_tm) = -sma.get_row(1) + INPUT.elem(0,0) * 
                                             input_direction_wts
    + NSLconv_copy(sma_sma_wts.get_row(2), SMA.get_row(0)) // from prep flex cells
    - NSLconv_copy(sma_sma_wts.get_row(1), SMA.get_row(3)) // from ext mvt cells
    + NSLconv_copy(vlo_sma_wts.get_row(0), VLO.get_row(1));
//    + NSLconv_copy(mc_sma_wts.get_row(0), MC.get_row(1));

// extension preparatory
  DIFF.eq(temp2, sma_tm) = -sma.get_row(2) + INPUT.elem(0,4) * input_center_wts
    + NSLconv_copy(vlo_sma_wts.get_row(0), VLO.get_row(2))
    + NSLconv_copy(sma_sma_wts.get_row(0), SMA.get_row(3)) // from mvt cells
    - NSLconv_copy(sma_sma_wts.get_row(1), SMA.get_row(0));// from prep flex cells
//    + NSLconv_copy(mc_sma_wts.get_row(0), MC.get_row(2));
// extension movement
  DIFF.eq(temp3, sma_tm) = -sma.get_row(3) + INPUT.elem(0,9) * 
                                             input_direction_wts
    + NSLconv_copy(sma_sma_wts.get_row(2), sma.get_row(2)) // from prep ext cells
    - NSLconv_copy(sma_sma_wts.get_row(1), sma.get_row(1)) // from flex mvt cells
    + NSLconv_copy(vlo_sma_wts.get_row(0), VLO.get_row(3));
//    + NSLconv_copy(mc_sma_wts.get_row(0), MC.get_row(3));
  
  sma = sma.set_row(temp0, 0);
  sma = sma.set_row(temp1, 1);
  sma = sma.set_row(temp2, 2);
  sma = sma.set_row(temp3, 3);

  SMA  = NSLramp(sma, sma_kx1, sma_ky1, sma_ky2);
}

// <a name="mc"><b>Motor Cortex</b> </a>
// <b>Click <a href="../model.html#mc">here</a> to read about assumptions.</b>
RUN_MODULE(bg_mc) {
  DIFF.eq(mc, mc_tm) = -mc
    + NSLconv_copy(sma_mc_wts, SMA)
    + NSLconv_copy(vlo_mc_wts, VLO);

  MC = NSLramp(mc, mc_kx1, mc_ky1, mc_ky2);
}

// <a name="put"><b>Putamen</b></a>
// <b>Click <a href="../model.html#put">here</a> to read about assumptions.</b>
// The putamen behaves similar to a bursting neuron, in that its output
// typically is a burst of APs at 40 Hz, for anywhere from 0.1 to 2.0 seconds.
// This is being modeled first by using a step function (since cells are either
// "on" or "off"), then using a hyperpolarization constant (see Wilson and
// Groves '81), as well as appropriate time constants.  For now, I'm just
// guessing at the hyperpolarization constant as 10.0, since it seems like it
// works.
// This was separated into the four rows representing the putamen, since I felt
// it wasn't appropriate to have all regions bursting for the same amount of
// time.

RUN_MODULE(bg_put) {
  temp0 = put.get_row(0);
  temp1 = put.get_row(1);
  temp2 = put.get_row(2);
  temp3 = put.get_row(3);

// flexion preparatory activity
  DIFF.eq(temp0, put_tm) = -put.get_row(0) 
    + NSLconv_copy(sma_put_wts.get_row(0), SMA.get_row(0)) 
    + NSLconv_copy(mc_put_wts.get_row(0), MC.get_row(0));

  put_flex_prep_on = NSLmax(temp0);
  if (put_flex_prep_on.elem() >= put_kx1.elem()) {
    if (burst_time_prep_flex.elem() == 0) {
      burst_time_prep_flex = drand48() * 1.9 + 0.1; // range is 0.1 to 2.0 sec
      time_passed_prep_flex = 0;
    } 
    else 
      time_passed_prep_flex.elem() = time_passed_prep_flex.elem() + delta; 
    if (time_passed_prep_flex.elem() >= burst_time_prep_flex.elem()) {
      time_passed_prep_flex = 0;
       burst_time_prep_flex = 0;
      temp0 = temp0 - put_hyperpol;
    }   
  }

// flexion movement activity 
  DIFF.eq(temp1, put_tm) = -put.get_row(1) 
    + NSLconv_copy(sma_put_wts.get_row(0), SMA.get_row(1))
    + NSLconv_copy(mc_put_wts.get_row(0), MC.get_row(1));
  put_flex_mvt_on = NSLmax(temp1);
  if (put_flex_mvt_on.elem() >= put_kx1.elem()) {
    if (burst_time_mvt_flex.elem() == 0) {
      burst_time_mvt_flex = drand48() * 1.9 + 0.1; // range is 0.1 to 2.0 sec
      time_passed_mvt_flex = 0;
    } 
    else 
      time_passed_mvt_flex.elem() = time_passed_mvt_flex.elem() + delta; 
    if (time_passed_mvt_flex.elem() >= burst_time_mvt_flex.elem()) {
      time_passed_mvt_flex = 0;
       burst_time_mvt_flex = 0;
      temp1 = temp1 - put_hyperpol;
    }   
  }

// extension preparatory activity
  DIFF.eq(temp2, put_tm) = -put.get_row(2) 
    + NSLconv_copy(sma_put_wts.get_row(0), SMA.get_row(2))
    + NSLconv_copy(mc_put_wts.get_row(0), MC.get_row(2));
  put_extend_prep_on = NSLmax(temp2);
  if (put_extend_prep_on.elem() >= put_kx1.elem()) {
    if (burst_time_prep_extend.elem() == 0) {
      burst_time_prep_extend = drand48() * 1.9 + 0.1; // range is 0.1 to 2.0 sec
      time_passed_prep_extend = 0;
    } 
    else 
      time_passed_prep_extend.elem() = time_passed_prep_extend.elem() + delta; 
    if (time_passed_prep_extend.elem() >= burst_time_prep_extend.elem()) {
      time_passed_prep_extend = 0;
       burst_time_prep_extend = 0;
      temp2 = temp2 - put_hyperpol;
    }   
  }

// extension movement activity
  DIFF.eq(temp3, put_tm) = -put.get_row(3) 
    + NSLconv_copy(sma_put_wts.get_row(0), SMA.get_row(3))
    + NSLconv_copy(mc_put_wts.get_row(0), MC.get_row(3));
  put_extend_mvt_on = NSLmax(temp3);
  if (put_extend_mvt_on.elem() >= put_kx1.elem()) {
    if (burst_time_mvt_extend.elem() == 0) {
      burst_time_mvt_extend = drand48() * 1.9 + 0.1; // range is 0.1 to 2.0 sec
      time_passed_mvt_extend = 0;
    } 
    else 
      time_passed_mvt_extend.elem() = time_passed_mvt_extend.elem() + delta; 
    if (time_passed_mvt_extend.elem() >= burst_time_mvt_extend.elem()) {
      time_passed_mvt_extend = 0;
       burst_time_mvt_extend = 0;
      temp3 = temp3 - put_hyperpol;
    }   
  }

  put = put.set_row(temp0, 0);
  put = put.set_row(temp1, 1);
  put = put.set_row(temp2, 2);
  put = put.set_row(temp3, 3);

  PUT = NSLstep(put, put_kx1, put_ky1, put_ky2);
}  

// <a name="gpe"><b>External Globus Pallidus</b></a>
// <b>Click <a href="../model.html#gpe">here</a> to read about assumptions.</b>
RUN_MODULE(bg_gpe) {
  DIFF.eq(gpe, gpe_tm) = -gpe
    - NSLconv_copy(put_gpe_wts, PUT)
    + gpe_tonic;

  GPE = NSLramp(gpe, gpe_kx1, gpe_ky1, gpe_ky2);
}

// <a name="stn"><b>Subthalamic Nucleus</b></a>
// <b>Click <a href="../model.html#stn">here</a> to read about assumptions.</b>
RUN_MODULE(bg_stn) {
  DIFF.eq(stn, stn_tm) = - stn
    - NSLconv_copy(gpe_stn_wts, GPE)
    + stn_tonic;

  STN = NSLramp(stn, stn_kx1, stn_ky2, stn_ky2);
}

// <a name="gpi"><b>Internal Globus Pallidus</b></a>
// <b>Click <a href="../model.html#gpi">here</a> to read about assumptions.</b>
RUN_MODULE(bg_gpi) {
  DIFF.eq(gpi, gpi_tm) = - gpi
    - NSLconv_copy(put_gpi_wts, PUT)
    + NSLconv_copy(stn_gpi_wts, STN)
    + gpi_tonic;

  GPI = NSLramp(gpi, gpi_kx1, gpi_ky1, gpi_ky2);
}

// <a name="snr"><b>Substantia Nigra pars Reticulata</b></a>
// <b>Click <a href="../model.html#snr">here</a> to read about assumptions.</b>
RUN_MODULE(bg_snr) {
  DIFF.eq(snr, snr_tm) = - snr
    - NSLconv_copy(put_snr_wts, PUT)
    + NSLconv_copy(stn_snr_wts, STN)
    + snr_tonic;

  SNR = NSLramp(snr, snr_kx1, snr_ky1, snr_ky2);
}

// <a name="vlo"><b>Ventrolateral pars Oralis of Thalamus</b></a>
// <b>Click <a href="../model.html#vlo">here</a> to read about assumptions.</b>
RUN_MODULE(bg_vlo) {
  DIFF.eq(vlo, vlo_tm) = - vlo
    - NSLconv_copy(snr_vlo_wts, SNR)
    - NSLconv_copy(gpi_vlo_wts, GPI)
    + vlo_tonic;

  VLO = NSLramp(vlo, vlo_kx1, vlo_ky1, vlo_ky2);
}

// <a name="mvt"><b>Flexion - Extension Movement Indicator</b></a>
RUN_MODULE(bg_mvt) {

for (int i = 0; i < 10; i++) {
temp4.elem(i) = mvt.elem(0);
temp5.elem(i) = mvt.elem(1);
}
// flexion movement
  DIFF.eq(temp4, mvt_tm) = - temp4 
    + NSLconv_copy(mc_mvt_wts.get_row(0), MC.get_row(1)) 
    + NSLconv_copy(vlo_mvt_wts.get_row(0), VLO.get_row(1));

// extension movement
  DIFF.eq(temp5, mvt_tm) = - temp5 
    + NSLconv_copy(mc_mvt_wts.get_row(0), MC.get_row(3)) 
    + NSLconv_copy(vlo_mvt_wts.get_row(0), VLO.get_row(3));

  mvt.elem(0) = temp4.sum() / NO_COLS;
  mvt.elem(1) = temp5.sum() / NO_COLS; 

  MVT = NSLstep(mvt, mvt_kx1, mvt_ky1, mvt_ky2);
  MVT_COL = MVT_COL.set_col(MVT, 0);
}


